<?php

namespace Drupal\Tests\lupus_decoupled\Functional;

use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\lupus_decoupled_ce_api\BaseUrlProviderTrait;
use Drupal\node\Entity\Node;
use Drupal\Tests\BrowserTestBase;

/**
 * Test Lupus Decoupled features.
 *
 * @group lupus_decoupled
 */
class LupusDecoupledApiResponseTest extends BrowserTestBase {

  use BaseUrlProviderTrait;

  /**
   * The node to use for testing.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $node;

  /**
   * Path to created node.
   *
   * @var string
   */
  protected $nodePath;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'content_translation',
    'custom_elements',
    'lupus_ce_renderer',
    'lupus_decoupled',
    'lupus_decoupled_ce_api',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create Basic page node type.
    $this->drupalCreateContentType(['type' => 'page', 'name' => 'Basic page']);
    $this->node = $this->drupalCreateNode();
    $this->nodePath = 'ce-api/node/' . $this->node->id();
  }

  /**
   * Tests if created node is accessible.
   */
  public function testExistingPageResponse() {
    $this->drupalGet('node/' . $this->node->id());
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Tests un-existing page access.
   */
  public function test404Page() {
    $this->drupalGet('i-dont-exist');
    $this->assertSession()->statusCodeEquals(404);
  }

  /**
   * Tests if created node is accessible at api endpoint.
   */
  public function testExistingPageApiResponse() {
    $this->drupalGet($this->nodePath);
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Tests if 'alternate' link has proper format.
   */
  public function testAlternateLink() {
    // As long as this is the only test that does translation: add language +
    // translation here instead of in setUp(). If the number of tests grows:
    // reconsider splitting out into translation specific test file.
    ConfigurableLanguage::createFromLangcode('de')->save();
    // Node needs reload before translation can be added.
    $this->node = Node::load($this->node->id());
    $this->node->addTranslation('de', ['title' => 'DE translation ' . $this->node->label()] + $this->node->toArray());
    $this->node->save();

    $response = json_decode($this->drupalGet($this->nodePath), TRUE);
    // 'alternate' must be in the 'link' section and point to the front end.
    $this->assertStringStartsWith(
      rtrim($this->getBaseUrlProvider()->getFrontendBaseUrl(), '/') . '/de/',
      $this->findLink($response['metatags']['link'] ?? [], 'alternate', 'de')
    );
  }

  /**
   * Finds a specific 'href' value in an array of 'link' metatags.
   *
   * @param array $link_metatags
   *   The array of 'link' metatags.
   * @param string $rel
   *   The 'rel' value to search for.
   * @param string $hreflang
   *   The 'hreflang' value to search for.
   *
   * @return string
   *   The corresponding 'href' value, or empty string if not found.
   */
  private function findLink(array $link_metatags, string $rel, string $hreflang) {
    foreach ($link_metatags as $link) {
      if (isset($link['href']) && isset($link['rel']) && $link['rel'] === $rel
          && isset($link['hreflang']) && $link['hreflang'] === $hreflang) {
        return $link['href'];
      }
    }

    return '';
  }

}
